<?php

/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\ExtAmastyGeoipRedirect\Controller\Change;

use Magento\Framework\App\ObjectManager;
use Magento\UrlRewrite\Model\UrlFinderInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;

/**
 * Responsible for loading page content.
 *
 * This is a basic controller that only loads the corresponding layout file. It may duplicate other such
 * controllers, and thus it is considered tech debt. This code duplication will be resolved in future releases.
 */
class Language extends \Magento\Framework\App\Action\Action {

    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $resultJsonFactory;
    protected $_helper;
    protected $_storeManager;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Store\Api\StoreCookieManagerInterface
     */
    protected $storeCookieManager;

    /** @var \Magento\Framework\App\Config\ScopeConfigInterface */
    protected $scopeConfig;

    /**
     * Language constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Store\Api\StoreCookieManagerInterface $storeCookieManager
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Store\Api\StoreCookieManagerInterface $storeCookieManager,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->_helper = $helper;
        $this->_storeManager = $storeManager;
        $this->customerSession = $customerSession;
        $this->storeCookieManager = $storeCookieManager;
        $this->scopeConfig = $scopeConfig;

        parent::__construct($context);
    }

    public function execute() {

        if ($this->getRequest()->isPost() && $this->getRequest()->getPost('store_id') && $this->getRequest()->getPost('country_id')) {

            $storeId = $this->getRequest()->getPost('store_id');
            $countryId = $this->getRequest()->getPost('country_id');
            $store = $this->_storeManager->getStore($storeId);
            $new_redirect_url = $store->getBaseUrl();

            if(!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_to_homepage')) {
                /** @var  \Magento\Framework\App\Response\RedirectInterface $redirect */
                $redirect = ObjectManager::getInstance()->get(\Magento\Framework\App\Response\RedirectInterface::class);
                /** @var  \Magento\UrlRewrite\Model\UrlFinderInterface $urlFinder */
                $urlFinder = ObjectManager::getInstance()->get(UrlFinderInterface::class);

                $refererUrl = $redirect->getRefererUrl();

                $actualUrl = null;
                foreach ($this->_storeManager->getStores() as $fromStore) {
                    $baseUrl = $fromStore->getBaseUrl();
                    if (strncasecmp($baseUrl, $refererUrl, strlen($baseUrl)) === 0) {
                        if($query_pos = strpos($refererUrl, '?')) {
                            $url_query_string = substr($refererUrl, $query_pos);
                            $refererUrl = str_replace($url_query_string, "", $refererUrl);
                        }
                        $actualUrl = $urlFinder->findOneByData(
                            [
                                'request_path' => str_replace($baseUrl, '', $refererUrl),
                                'store_id'     => $fromStore->getId()
                            ]
                        );

                        break;
                    }
                }

                if ($actualUrl) {
                    $destUrl = $urlFinder->findOneByData(
                        [
                            'store_id'         => $store->getId(),
                            'entity_id'        => $actualUrl->getEntityId(),
                            'entity_type'      => $actualUrl->getEntityType(),
                            'is_autogenerated' => 1
                        ]
                    );

                    $new_redirect_url = $store->getBaseUrl() . $destUrl->getRequestPath();
                }
            }

            $this->_helper->setCountryCookie($countryId);
            $this->_helper->setStoreCookie($storeId);
            $this->storeCookieManager->setStoreCookie($store);

            $resultJson = $this->resultJsonFactory->create();

            $resultJson->setData(["url" => $new_redirect_url]);

            return $resultJson;
        } else {
            return null;
        }
    }

}
