<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Helper;

use Amasty\Geoip\Model\Geolocation;
use FiloBlu\ExtAmastyGeoipRedirect\Cookie\CanLog;
use FiloBlu\ExtAmastyGeoipRedirect\Cookie\Country;
use FiloBlu\ExtAmastyGeoipRedirect\Cookie\Currency;
use FiloBlu\ExtAmastyGeoipRedirect\Cookie\LocalizedCountry;
use FiloBlu\ExtAmastyGeoipRedirect\Cookie\Misfits;
use FiloBlu\ExtAmastyGeoipRedirect\Cookie\Store;
use FiloBlu\ExtAmastyGeoipRedirect\Cookie\Url;
use FiloBlu\ExtAmastyGeoipRedirect\Model\Source\Logic;
use Magento\Directory\Model\CountryFactory;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\InputException;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\Stdlib\Cookie\CookieSizeLimitReachedException;
use Magento\Framework\Stdlib\Cookie\FailureToSendException;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Data
 * @package FiloBlu\ExtAmastyGeoipRedirect\Helper
 */
class Data extends AbstractHelper
{
    /**
     * @var CountryFactory
     */
    protected $countryFactory;

    /**
     * @var Geolocation
     */
    protected $geolocation;

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var
     */
    protected $geoData;

    /**
     * @var \FiloBlu\MultiDomain\Helper\Data
     */
    protected $multiDomainHelper;

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var Country
     */
    protected $countryCookie;

    /**
     * @var LocalizedCountry
     */
    protected $localizedCountry;

    /**
     * @var Url
     */
    protected $cookieUrl;

    /**
     * @var Currency
     */
    protected $cookieCurrency;

    /**
     * @var Store
     */
    protected $cookieStore;

    /**
     * @var Misfits
     */
    protected $cookieMisfits;

    /**
     * @var CanLog
     */
    protected $cookieCanLog;

    /**
     * Data constructor.
     * @param Context $context
     * @param CountryFactory $countryFactory
     * @param Geolocation $geolocation
     * @param StoreManagerInterface $storeManager
     * @param Url $cookieUrl
     * @param \FiloBlu\MultiDomain\Helper\Data $multiDomainHelper
     * @param Currency $cookieCurrency
     * @param Store $cookieStore
     * @param Misfits $cookieMisfits
     * @param CanLog $cookieCanLog
     * @param Country $countryCookie
     * @param LocalizedCountry $localizedCountry
     */
    public function __construct(
        Context $context,
        CountryFactory $countryFactory,
        Geolocation $geolocation,
        StoreManagerInterface $storeManager,
        Url $cookieUrl,
        \FiloBlu\MultiDomain\Helper\Data $multiDomainHelper,
        Currency $cookieCurrency,
        Store $cookieStore,
        Misfits $cookieMisfits,
        CanLog $cookieCanLog,
        Country $countryCookie,
        LocalizedCountry $localizedCountry

    ) {
        $this->countryFactory = $countryFactory;
        $this->geolocation = $geolocation;
        $this->remoteAddress = $context->getRemoteAddress();
        $this->storeManager = $storeManager;
        $this->multiDomainHelper = $multiDomainHelper;
        $this->request = $context->getRequest();
        $this->countryCookie = $countryCookie;
        $this->localizedCountry = $localizedCountry;
        $this->cookieUrl = $cookieUrl;
        $this->cookieCurrency = $cookieCurrency;
        $this->cookieStore = $cookieStore;
        $this->cookieMisfits = $cookieMisfits;
        $this->cookieCanLog = $cookieCanLog;
        parent::__construct($context);
    }

    public function getGeoData()
    {
        if (!$this->geoData) {
            $currentStoreId = $this->storeManager->getStore()->getId();
            $scopeStores = ScopeInterface::SCOPE_STORES;

            $currentIp = $this->remoteAddress->getRemoteAddress();
            $fakeIp = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_ip', $scopeStores, $currentStoreId);
            $fakeCountry = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_country', $scopeStores, $currentStoreId);

            if ($fakeIp) {
                $currentIp = $fakeIp;
            }

            $country_header_config = $this->scopeConfig->getValue('amgeoipredirect/restriction/geolocation_header');

            if (!$this->request->getHeader($country_header_config)) {
                $location = $this->geolocation->locate($currentIp);
                $country = $location->getCountry();
            } else {
                $country = $this->request->getHeader($country_header_config);
            }

            /* $location = $this->geolocation->locate($currentIp);
            $country  = $location->getCountry(); */

            if ($fakeCountry) {
                $country = $fakeCountry;
            }

            $this->geoData = new DataObject();

            if ($country) {
                $this->geoData->setCountryCode($country);
                $this->geoData->setCountryName($this->getCountryName($country, 'en_US'));

                $allStores = $this->storeManager->getStores();

                $currentGroup = $this->multiDomainHelper->getCurrentGroup();

                foreach ($allStores as $store) {
                    if (!$store->isActive()) {
                        continue;
                    }

                    if ($currentGroup && $currentGroup !== $this->multiDomainHelper->getCurrentGroup($store->getWebsite()->getCode())) {
                        continue;
                    }

                    $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', $scopeStores, $store->getId());

                    if ($countries && strpos($countries, $country) !== false) {
                        $this->geoData->setStore($store);
                        break;
                    }
                }
            }

            return $this->geoData;
        }

        return $this->geoData;
    }

    /**
     * @param string|null $iso
     * @param string|null $locale
     * @return mixed
     */
    public function getCountryName($iso = null, $locale = null)
    {
        $country = $this->countryFactory->create()->loadByCode($iso);
        return $country->getName($locale);
    }

    /**
     * @return mixed
     */
    public function getAllCountries()
    {
        return $this->countryFactory->create()->getCollection()->getData();
    }

    /**
     * @return string
     */
    public function getCountryCookieName()
    {
        return Country::COOKIE_NAME;
    }

    /**
     * @param $country
     * @throws CookieSizeLimitReachedException
     * @throws FailureToSendException
     * @throws InputException
     */
    public function setLocalizedCountryCookie($country)
    {
        $this->localizedCountry->set($country);
    }

    /**
     * @return string
     */
    public function getLocalizedCountryCookie()
    {
        return $this->localizedCountry->get();
    }

    /**
     * @return string
     */
    public function getLocalizedCountryCookieName()
    {
        return LocalizedCountry::COOKIE_NAME;
    }

    /**
     * @param $storeId
     * @throws CookieSizeLimitReachedException
     * @throws FailureToSendException
     * @throws InputException
     */
    public function setStoreCookie($storeId)
    {
        $this->cookieStore->set($storeId);
    }

    /**
     * @return string
     */
    public function getStoreCookie()
    {
        return $this->cookieStore->get();
    }

    /**
     * @return string
     */
    public function getMisfitsCookieName()
    {
        return Misfits::COOKIE_NAME;
    }

    /**
     *
     */
    public function setMisfitsCookie()
    {
        $this->cookieMisfits->set(1);
    }

    /**
     * @return string
     */
    public function getMisfitsCookie()
    {
        return $this->cookieMisfits->get(1);
    }

    /**
     * @return string
     */
    public function getStoreCookieName()
    {
        return Store::COOKIE_NAME;
    }

    /**
     *
     */
    public function setCurrencyCookie()
    {
        $this->cookieCurrency->set(1);
    }

    /**
     * @return string
     */
    public function getCurrencyCookie()
    {
        return $this->cookieCurrency->get();
    }

    /**
     * @return string
     */
    public function getCurrencyCookieName()
    {
        return Currency::COOKIE_NAME;
    }

    /**
     *
     */
    public function setUrlCookie()
    {
        $this->cookieUrl->set(1);
    }

    /**
     * @return string
     */
    public function getUrlCookie()
    {
        return $this->cookieUrl->get();
    }

    /**
     * @return string
     */
    public function getUrlCookieName()
    {
        return Url::COOKIE_NAME;
    }

    /**
     * @return string
     */
    public function getCanLogCookie()
    {
        return $this->cookieCanLog->get();
    }

    /**
     * @param null $locale
     * @return mixed
     * @throws CookieSizeLimitReachedException
     * @throws FailureToSendException
     * @throws InputException
     */
    public function getCurrentCountryName($locale = null)
    {
        return $this->getCountryName($this->getCurrentCountryCode(), $locale);
    }

    /**
     * @return string
     * @throws CookieSizeLimitReachedException
     * @throws FailureToSendException
     * @throws InputException
     */
    public function getCurrentCountryCode()
    {
        $current = $this->countryCookie->get();
        $new = $this->scopeConfig->getValue('general/country/default', ScopeInterface::SCOPE_STORE);

        if ($current === $new) {
            return $current;
        }

        $this->countryCookie->set($new);
        return $new;
    }

    /**
     * @return string
     */
    public function getCountryCookie()
    {
        return $this->countryCookie->get();
    }

    /**
     * @param $country
     * @throws InputException
     * @throws CookieSizeLimitReachedException
     * @throws FailureToSendException
     */
    public function setCountryCookie($country)
    {
        $this->countryCookie->set($country);
    }

    /**
     * @param $request
     * @param $logEnabled
     * @param $currentUrl
     * @param $logger
     * @return bool
     */
    public function applyLogic($request, $logEnabled, $currentUrl, $logger)
    {
        $applyLogic = $this->scopeConfig->getValue('amgeoipredirect/restriction/apply_logic');

        $redirectAllowed = false;

        if ($this->geolocationRequest($request)) {
            return true;
        }

        switch ($applyLogic) {

            case Logic::SPECIFIED_URLS:
                $acceptedUrls = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/accepted_urls'));
                foreach ($acceptedUrls as $url) {
                    $url = trim($url);
                    if ($url && $currentUrl && strpos($currentUrl, $url) !== false) {
                        return true;
                    }
                }
                break;
            case Logic::EXCEPT_URLS:
                $exceptedUrls = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/excepted_urls'));
                foreach ($exceptedUrls as $url) {
                    $url = trim($url);
                    if ($url && $currentUrl && strpos($currentUrl, $url) !== false) {
                        return false;
                    }

                    $redirectAllowed = true;
                }
                break;
            case Logic::HOMEPAGE_ONLY:
                $routeName = $request->getRouteName();
                $action = $request->getActionName();
                if ($routeName === 'cms' && $action === 'index') {
                    $redirectAllowed = true;
                }
                break;
            case Logic::SPECIFIED_PATH:
                $acceptedPath = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/accepted_path'));
                $currentPath = parse_url($currentUrl, PHP_URL_PATH);

                if ($logEnabled) {
                    $logger->info("Current path {$currentPath}");
                }

                foreach ($acceptedPath as $path) {
                    $path = trim($path);
                    if ($currentPath && $path && $path === $currentPath) {
                        return true;
                    }
                }
                break;
            // case Logic::ALL_URLS:
            default:
                return true;
        }

        return $redirectAllowed;
    }

    /**
     * @param $request
     * @return bool
     */
    public function geolocationRequest($request)
    {
        return (null !== $request->getParam('geolocation') && (int)$request->getParam('geolocation') === 1);
    }

    /**
     * @param $request
     * @return bool
     */
    public function notGeolocalizedRequest($request)
    {
        return (null !== $request->getParam('notgeolocalized') && (int)$request->getParam('notgeolocalized') === 1);
    }
}
